require 'forwardable'
require 'gemsupport'
require 'start_her/configuration'
require 'start_her/logger'
require 'start_her/retry_policies'
require 'start_her/refinements'
require 'start_her/utils'

module StartHer
  module_function

  extend SingleForwardable

  attr_accessor :config

  def config
    @config ||= Configuration.new
  end
  delegate logger: :config

  def configure
    yield config
  end

  def env
    (ENV['RACK_ENV'] || ENV['RAILS_ENV'] || 'development').tap do |e|
      fail 'Invalid environment name' unless %w(development test staging production).include?(e)
    end
  end

  # Safe loading for stuff that needs StartHer#config
  autoload :RedisClient, 'start_her/redis_client'
  autoload :Subscriber, 'start_her/subscriber'
  autoload :Publisher, 'start_her/publisher'
  autoload :Heartbeat, 'start_her/heartbeat'
end
