module StartHer
  class Heartbeat
    include StartHer::Publisher
    include StartHer::Utils

    attr_reader :request, :options
    attr_accessor :response

    CHANNELS = { in: 'hb_ping', out: 'hb_pong' }
    DEFAULT_OPTS = { heartbeat: CHANNELS }

    def self.call(request, service_klass, opts = {}, &subscriber_heartbeat_block)
      new(request, service_klass, opts, &subscriber_heartbeat_block).call
    end

    def initialize(request, service_klass, opts = {}, &subscriber_heartbeat_block)
      @request = request
      @subscriber_heartbeat_block = subscriber_heartbeat_block
      @options = DEFAULT_OPTS.merge opts
      @response = {
        id: msid(service_klass),
        generated_at: Time.now.utc.to_s,
        service_name: service_klass.to_s,
        ping: request['data']
      }
      @response[:version] = service_klass::VERSION if defined? service_klass::VERSION
    end

    def call
      @subscriber_heartbeat_block.call(response) if @subscriber_heartbeat_block
      publish(options[:heartbeat][:out],
        id: SecureRandom.uuid, generated_at: Time.now.utc.to_s, data: response)
    end
  end
end
