module StartHer
  module Testing
    module_function

    attr_accessor :__test_mode

    # rubocop:disable Style/GlobalVars
    def clear_stubed_redis
      $stubed_redis.clear_db if $stubed_redis
    end
    # rubocop:enable Style/GlobalVars

    # rubocop:disable Style/TrivialAccessors
    def __set_test_mode(mode)
      @__test_mode = mode
    end
    # rubocop:enable Style/TrivialAccessors

    def disable!
      __set_test_mode(:disable)
    end

    def fake!
      __set_test_mode(:fake)
    end

    def disable?
      @__test_mode == :disable
    end

    def fake?
      @__test_mode == :fake
    end
  end
  Testing.fake!

  class StubedRedis
    attr_reader :namespace, :db

    def initialize(namespace:)
      @namespace = namespace
      @db = {
        lists:  {},
        lists_expiration:  {},
        channels: {},
        strings: {}
      }
    end

    def clear_db
      @db = {
        lists:  {},
        lists_expiration:  {},
        channels: {},
        strings: {}
      }
    end

    # Keys command
    def multi
      yield self
    end

    # Keys command
    def keys(pattern)
      lists.keys.select { |key| key.match(pattern.gsub '**', '*') }
    end

    # Keys command
    def ttl(key)
      if lists_expiration[key]
        lists_expiration[key]
      elsif lists[key] || channels[key]
        -1
      else
        -2
      end
    end

    # Keys command
    def expire(key, ttl)
      lists_expiration[key] = ttl
    end

    # String command
    def get(key)
      strings[key]
    end

    # String command
    def getset(key, value)
      tmp = strings[key]
      strings[key] = value
      tmp
    end

    def setnx(key, value)
      if strings[key]
        0
      else
        strings[key] = value
        1
      end
    end

    # List command
    def lpush(list, data)
      (lists[fullkey(list)] ||= []).insert(0, data)
    end

    # List command
    def lrange(list, min, max)
      lists[fullkey(list)][min..max]
    end

    # PubSub command
    # rubocop:disable Metric/AbcSize,Style/MultilineOperationIndentation
    def publish(channel, message)
      return unless channels[fullkey(channel)] && channels[fullkey(channel)].callbacks &&
        channels[fullkey(channel)].callbacks[:pmessage]
      channels[fullkey(channel)].callbacks[:pmessage].call(channel, channel, message)
    end
    # rubocop:enable Metric/AbcSize,Style/MultilineOperationIndentation

    # PubSub command
    # rubocop:disable Metrics/AbcSize
    def psubscribe(*channels)
      channels.each do |channel|
        db[:channels][fullkey(channel)] = Subscription.new
        yield db[:channels][fullkey(channel)]
        db[:channels][fullkey(channel)].callbacks[:psubscribe].call(channel, 0)
      end
    end
    # rubocop:enable Metrics/AbcSize

    class Subscription
      attr_reader :callbacks

      def initialize
        @callbacks = {}
      end

      def psubscribe(&block)
        callbacks[:psubscribe] = block
      end

      def pmessage(&block)
        callbacks[:pmessage] = block
      end
    end

    private

    def fullkey(key)
      if namespace
        "#{namespace}:#{key}"
      else
        key
      end
    end

    def lists
      db[:lists]
    end

    def lists_expiration
      db[:lists_expiration]
    end

    def channels
      db[:channels]
    end

    def strings
      db[:strings]
    end
  end

  module RedisClient
    alias_method :real_client, :client
    # rubocop:disable Style/GlobalVars
    def client(options = {})
      if Testing.fake?
        options = client_opts(options)
        $stubed_redis ||= StubedRedis.new(namespace: options[:namespace])
      else
        real_client(options)
      end
    end
    # rubocop:enable Style/GlobalVars
  end
end
